'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setProcessLogToGrid = exports.getGridAndScreen = undefined;

var _blessed = require('blessed');

var _blessed2 = _interopRequireDefault(_blessed);

var _blessedContrib = require('blessed-contrib');

var _blessedContrib2 = _interopRequireDefault(_blessedContrib);

var _spawnly = require('spawnly');

var _spawnly2 = _interopRequireDefault(_spawnly);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getGridAndScreen = exports.getGridAndScreen = function getGridAndScreen() {
  var screen = _blessed2.default.screen();
  var grid = new _blessedContrib2.default.grid({ rows: 12, cols: 12, screen: screen }); // eslint-disable-line

  return {
    screen: screen,
    grid: grid
  };
};

var getPositionByIndex = function getPositionByIndex(index) {
  var positions = [{ row: 0, col: 0 }, { row: 0, col: 4 }, { row: 0, col: 8 }, { row: 6, col: 0 }, { row: 6, col: 4 }, { row: 6, col: 8 }];

  return positions[index];
};

var setProcessLogToGrid = exports.setProcessLogToGrid = function setProcessLogToGrid(cmd, grid, index) {
  var _getPositionByIndex = getPositionByIndex(index),
      row = _getPositionByIndex.row,
      col = _getPositionByIndex.col;

  var box = grid.set(row, col, 6, 4, _blessed2.default.box, {
    label: cmd.substr(0, 40),
    padding: { top: 0, left: 0, right: 0, bottom: 0 },
    border: {
      type: 'line'
    },
    style: {
      border: {
        fg: 'yellow'
      }
    }
  });

  var log = _blessed2.default.log({
    parent: box,
    tags: true,
    width: '100%-5',
    scrollable: true,
    input: true,
    alwaysScroll: true,
    scrollbar: {
      ch: ' ',
      inverse: true
    },
    keys: true,
    vi: true,
    mouse: true
  });

  var addListener = function addListener(stream) {
    if (!stream.readable) return;
    stream.on('data', function (chunk) {
      return log.log(chunk.toString());
    });
  };

  var cp = void 0;
  return {
    start: function start() {
      cp = (0, _spawnly2.default)(cmd, { stdio: 'pipe' });

      addListener(cp.stdout);
      addListener(cp.stderr);

      cp.on('close', function (exitCode) {
        log.log('process exit with code ' + exitCode);
      });
    },
    stop: function stop() {
      if (cp && !cp.exitCode) {
        cp.stdout.removeAllListeners('data');
        cp.stderr.removeAllListeners('data');
        cp.removeAllListeners('close');
        cp.kill('SIGINT');
      }
    }
  };
};