'use strict';

var _grid = require('./grid');

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var exec = require('child_process').exec;
var path = require('path');
var nodeProcess = require('./process');
var domain = require('domain');
var manager = require('../index');

var addNPMBinToPath = function addNPMBinToPath(cb) {
  return new Promise(function (resolve, reject) {
    exec('npm bin', function (error, stdout) {
      if (error) {
        cb && cb(error);
        reject(error);
        return;
      }

      if (typeof process.env.FORCE_COLOR === 'undefined') {
        process.env.FORCE_COLOR = 'true';
      }

      process.env.PATH += '' + path.delimiter + stdout.trim();

      cb && cb();
      resolve();
    });
  });
};

var printFailed = function printFailed(entries) {
  return entries.reduce(function (seq, entry) {
    seq += '   - cmd: ' + entry.cmd + ', exitCode: ' + entry.exitCode + '\n';
    return seq;
  }, '\n');
};

module.exports = {
  _executeCommandsInDashboard: function _executeCommandsInDashboard(program, cmds) {
    var _getGridAndScreen = (0, _grid.getGridAndScreen)(),
        grid = _getGridAndScreen.grid,
        screen = _getGridAndScreen.screen;

    var commands = cmds.reduce(function (seq, cmd, index) {
      var command = (0, _grid.setProcessLogToGrid)(cmd, grid, index);
      command.start();
      seq.push(command);
      return seq;
    }, []);

    screen.key(['escape', 'q', 'C-c'], function () {
      commands.forEach(function (cmd) {
        return cmd.stop();
      });
      return process.exit(0); // eslint-disable-line
    });

    screen.render();
  },
  _execute: function _execute(program, cmds) {
    var cmdManager = manager.create(); // eslint-disable-line

    var opts = program.opts;

    cmdManager.on('command:start', function (e, args) {
      program.subtle('starting command', args.cmd);
    });

    cmdManager.on('command:error', function (e, args) {
      program.subtle('command error', args, 'duration: ', args.durationFormatted);
    });

    cmdManager.on('command:exit', function (e, args) {
      var method = args.exitCode === 0 ? 'subtle' : 'warn';
      program[method]('command', args.cmd, 'exited with code', args.exitCode + ', took:', args.durationFormatted);
      if (opts.sortOutput) {
        args.stdout && console.log(args.stdout);
        args.stderr && console.error(args.stderr);
      }
      if (args.exitCode !== 0 && opts.bail) {
        program.warn('command', args.cmd, 'failed. Stopping all');
        cmdManager.stopAll();
        process.exit(1); // eslint-disable-line
      }
    });

    cmdManager.on('command:killed', function (e, args) {
      program.subtle('command killed:', args.cmd, 'pid:', args.pid);
    });

    var d = domain.create();

    d.on('error', function () {
      cmdManager.stopAll();
    });

    var p = cmdManager.runCmds(cmds, {
      stdio: opts.sortOutput ? 'pipe' : 'inherit'
    });

    p.then(function (args) {
      var results = [];
      args.forEach(function (result) {
        if (result.exitCode !== 0) {
          results.push({
            cmd: result.cmd,
            exitCode: result.exitCode
          });
        }
      });

      if (results.length > 0) {
        program.error('Some commands failed', '\n', printFailed(results));
        process.exit(1); // eslint-disable-line
      }
    });

    var lines = '\n    Commands execution started\n\n    To kill commands from the shell In case I become a zombie, execute:\n\n    ' + cmdManager.getKillCommand() + '\n\n    ';

    program.subtle(lines);

    nodeProcess.on('SIGINT', function (code) {
      program.subtle('killing all processes');
      cmdManager.stopAll();
      nodeProcess.exit(code);
    });
  },
  run: function run(program) {
    var _this = this;

    return _asyncToGenerator(regeneratorRuntime.mark(function _callee() {
      var opts, error, showHelp, cmds, err, stderr;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              opts = program.opts, error = program.error, showHelp = program.showHelp;
              cmds = opts._;

              if (!(cmds.length === 0)) {
                _context.next = 6;
                break;
              }

              error('please provide some commands to execute');
              showHelp();
              return _context.abrupt('return');

            case 6:
              _context.prev = 6;
              _context.next = 9;
              return addNPMBinToPath();

            case 9:
              _context.next = 18;
              break;

            case 11:
              _context.prev = 11;
              _context.t0 = _context['catch'](6);
              err = _context.t0.error;
              stderr = _context.t0.stderr;

              error('received error', err);
              stderr && error(stderr);
              process.exit(1); // eslint-disable-line

            case 18:

              if (opts.dashboard) {
                _this._executeCommandsInDashboard(program, cmds);
              } else {
                _this._execute(program, cmds);
              }

            case 19:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this, [[6, 11]]);
    }))();
  }
};